ArrayList<Video> videosConstruction(Table table) {
  ArrayList<Video> videos = new ArrayList<Video>();
  int numRows = table.getRowCount();

  for (int i=0; i<numRows; i++) {
    String title = table.getString(i, 1);
    String channelTitle = table.getString(i, 2);
    int categoryId = table.getInt(i, 3);
    int view_count = table.getInt(i, 4);
    int response_count = table.getInt(i, 5);
    String published_date = table.getString(i, 6);
    String published_time = table.getString(i, 7);
    String trending_date = table.getString(i, 8);

    Video newVideo = new Video(title, channelTitle, categoryId, view_count, response_count, published_date, published_time, trending_date);
    videos.add(newVideo);
  }
  return videos;
}

int[] viewCounts(ArrayList<Video> videos) {
  int[] viewCounts = new int[videos.size()];
  for (int i = 0; i < videos.size(); i++) {
    Video video = videos.get(i);
    viewCounts[i] = video.view_count;
  }
  return viewCounts;
}


PVector pointCoordinate(String published_date, String published_time, String trending_date, int view_count) {
  float point_x, point_y, point_z;

  String[] published_dates = split(published_date, '-');
  int published_year = int(published_dates[0]);
  int published_month = int(published_dates[1]);
  int published_day = int(published_dates[2]);

  String[] trending_dates = split(trending_date, '-');
  //int trending_year = int(trending_dates[0]);
  int trending_month = int(trending_dates[1]);
  int trending_day = int(trending_dates[2]);

  String[] times = split(published_time, ':');
  float hour = float(times[0]);
  float minute = float(times[1]);
  float second = float(times[2]);

  if (published_year == 2021) {
    point_x = map(((published_month - 1) * 30 + published_day), 0, 365, -200, 200) ;
  } else {
    point_x = map(((published_month - 1) * 30 + published_day - 360), -20, 365, -200, 200);
  }

  point_y = map((hour * 60 + minute + second/60), 0, 1440, -200, 200) ;
  //point_z = map(((trending_month - 1) * 30 + trending_day), 0, 365, -200, 200);
  point_z = map(log(view_count), log(60000), log(89070000), -600, 600);
  //point_z = log(view_count);

  PVector pCoordinate = new PVector(point_x, point_y, point_z);

  return pCoordinate;
}

color colorDetermine(int categoryId) {
  color c = #FFFFFF;
  if (categoryId == 26) {
    c = #eb3493; 
  } else if (categoryId == 27) {
    c = #eb8634; 
  } else if (categoryId == 28) {
    c = #34ebbd; 
  }
  return c;
}

void drawPoints (ArrayList<Video> videos) {
  //int[] viewCounts = new int[videos.size()];
  for (int i = 0; i < videos.size(); i++) {
    Video video = videos.get(i);
    if (video.published_date.startsWith("2020") || video.view_count == 0 || video.response_count <= 5) {
      continue;
    }
    //viewCounts[i] = video.view_count;
    PVector pCoordinate = pointCoordinate(video.published_date, video.published_time, video.trending_date, video.view_count);
    float x, y, z;
    x = pCoordinate.x;
    y = pCoordinate.y;
    z = pCoordinate.z;

    float d_x = screenX(x, y, z);
    float d_y = screenY(x, y, z);

    float distanceObjectMouse = sq(d_x - mouseX) + sq(d_y-mouseY);
    color c = colorDetermine(video.categoryId);

    float radius = map(log(video.response_count), log(1802), log(2404748), 1, 10);

    if (distanceObjectMouse < 20) {
      strokeWeight(12);
    } else {
      strokeWeight(4);
    }

    if (showPoint == true) {
      stroke(c);
      point(x, y, z);
    }

    if (showResponse == true) {
      line(x, y, z, x, y, z + radius);
    }

    if (distanceObjectMouse < 20) {
      fill(255);
      textSize(20);
      pushMatrix();
      translate(x, y, z);
      scale(0.4);
      textAlign(LEFT, CENTER);
      text("Title: " + video.title + "\nPublished Date: " + video.published_date + "\nPublished_time: " + video.published_time + "\n" + "View Count:" + video.view_count + "\n" + "Response Count:" + video.response_count + "\nChannel: " + video.channelTitle + "\nTrending Date: " + video.trending_date, 10, 0, 0);
      popMatrix();
    }
  }  
}

void drawLabels(float z) {
  fill(255);

  textSize(30);

  pushMatrix();
  translate(-200, -200, z);
  scale(0.5);
  textAlign(RIGHT, TOP);
  text("2021-01-01", 0, -30, 0);
  popMatrix();

  pushMatrix();
  translate(0, -200, z);
  scale(0.5);
  textAlign(CENTER, CENTER);
  text("2021-06-01", 0, -30, 0);
  popMatrix();

  pushMatrix();
  translate(200, -200, z);
  scale(0.5);
  textAlign(LEFT, TOP);
  text("2021-12-31", 0, -30, 0);
  popMatrix();


  pushMatrix();
  translate(-200, -200, z);
  scale(0.5);
  textAlign(RIGHT, TOP);
  text("00:00", -5, 0, 0);
  popMatrix();

  pushMatrix();
  translate(-200, 0, z);
  scale(0.5);
  textAlign(RIGHT, CENTER);
  text("12:00", -5, 0, 0);
  popMatrix();

  pushMatrix();
  translate(-200, 200, z);
  scale(0.5);
  textAlign(RIGHT, BOTTOM);
  text("24:00", -5, 0, 0);
  popMatrix();
}

void drawChannelLines(String channel_title, ArrayList<Video> videos) {
  float x, y, z;
  beginShape();
  for (int i = 0; i < videos.size()-1; i++) {
    Video video = videos.get(i);
    //Video video2 = videos.get(i+1);
    if (video.published_date.startsWith("2020") || video.view_count == 0 || video.response_count == 0) {
      continue;
    }

    if (video.channelTitle.contains(channel_title) || video.channelTitle == channel_title) {
      PVector p_coordinate = pointCoordinate(video.published_date, video.published_time, video.trending_date, video.view_count);
      x = p_coordinate.x;
      y = p_coordinate.y;
      z = p_coordinate.z;
      noFill();
      strokeWeight(1);
      curveVertex(x, y, z);
    }
  }
  endShape();
}

ArrayList<String> creat_channels(ArrayList<Video> videos) {
  ArrayList<String> channels = new ArrayList<String>();
  for (int i = 0; i < videos.size(); i++) {
    Video video = videos.get(i);
    if (video.published_date.startsWith("2020") || video.view_count == 0 || video.response_count == 0) {
      continue;
    }
    if (!channels.contains(video.channelTitle)) {
      channels.add(video.channelTitle);
    }
  }
  return channels;
}

//ArrayList<String> creat_titles(ArrayList<Video> videos) {
//  ArrayList<String> titles = new ArrayList<String>();
//  for (int i = 0; i < videos.size(); i++) {
//    Video video = videos.get(i);
//    if (video.published_date.startsWith("2020") || video.view_count == 0 || video.response_count == 0) {
//      continue;
//    }
//    if (!titles.contains(video.title)) {
//      titles.add(video.title);
//    }
//  }
//  return titles;
//}

//void drawTitleLines(String title, ArrayList<Video> videos) {
//  float x, y, z;
//  beginShape();
//  for (int i = 0; i < videos.size()-1; i++) {
//    Video video = videos.get(i);
//    //Video video2 = videos.get(i+1);
//    if (video.published_date.startsWith("2020") || video.view_count == 0 || video.response_count == 0) {
//      continue;
//    }

//    if (video.title == title) {
//      PVector p_coordinate = pointCoordinate(video.published_date, video.published_time, video.trending_date, video.view_count);
//      x = p_coordinate.x;
//      y = p_coordinate.y;
//      z = p_coordinate.z;
//      noFill();
//      strokeWeight(1);
//      curveVertex(x, y, z);
//    }
//  }
//  endShape();
//}
